<?php

namespace Teknisa\ACL\Util;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DataSource\FilterCriteria;

class AclUtilities {
    const GERAL_ENTITY_DIR = '\Teknisa\Model\Geral\Entities\\';
    
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            //(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
            if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values)) {
                        if(is_array($values[0]) && isset($values[0]['_ALL'])) {
                            $values = implode(",", array_values($values[0]));
                        } else {
                            $values = "('" . implode("','", $values) . "')";
                        }
                    }
                    if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                        $operator = 'LIKE';
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL' || $operator == 'MAPPED_LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode(' OR ', $likes);
                        $filterSQL .= " AND ( $filterLikeAll )";
                    } else if ($operator == 'LIKE') {
                        $filterSQL .= " AND $name $operator $values ";
                    } else if ($operator == 'IS_NULL') {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name IS $values ";
                        } else {
                            $filterSQL .= " AND $name IS $values ";
                        }
                    } else if ($operator == '='){
                        if(!empty($tableAlias)){
                            $filterSQL .= " AND $tableAlias.$name $operator '$values' ";
                        } else {
                            $filterSQL .= " AND $name $operator '$values' ";
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values ";
                        } else {
                            $filterSQL .= " AND $name $operator $values ";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }
    
    public static function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    public static function getDefaultRuleParameter() {
        $defaultRule = null;

        try {
            return InstanceManager::getInstance()->getParameter('defaultRule');
        } catch(\Exception $e) { }

        return $defaultRule;
    }

    public static function getFilterTypeIn(array $items, $field)
    {
        return ' AND ' . $field . ' IN (' . implode(",", $items) . ')';
    }

    public static function createNewFilterCriteria($conditions, $dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName);
        foreach($conditions as $condition) {
            $name = isset($condition['name']) ? $condition['name'] : $condition['columnName'];
            if($condition['operator'] === 'ORDER_BY') {
                $filterCriteria->addOrderBy($name['name'], $name['direction']);
            } else {
                $filterCriteria->addCondition($name, $condition['operator'], $condition['value']);
            }
        }
        return $filterCriteria;
    }

    public static function fixBugWhereINQuery(array $ids, $column, $isNotIn = false, $isString = false)
    {
        $arrays = array_chunk($ids, 900);
        $allProjectsString = '';
        $in = $isNotIn ? 'NOT IN' : 'IN';
        $connection = $isNotIn ? 'AND' : 'OR';
        if ($isString) {
            $glue = "','";
            $start = "'";
            $end = "'";
        } else {
            $glue = ',';
            $start = '';
            $end = '';
        }
        foreach ($arrays as $array) {
            $allProjectsString .= " $column $in (" . self::getStringForArray($array, $glue, $start, $end) . ") $connection";
        }
        $allProjectsString .= " $column $in ($start 0 $end) ";
        return $allProjectsString;
    }

    public static function getStringForArray($array,$glue = ',',$start = '',$end = '',$default = ''){
        $return = $default;
        if(!empty($array)) {
            $return = $start . implode($glue, $array) . $end;
        }
        return $return;
    }

    public static function getArrayFromAssociativeArray( array $array = array(), $key = 'ID')
    {
        $result = array();
        foreach ($array as $row) {
            $result[] = $row[$key] ;
        }
        return $result;
    }
} 